// @dart = 2.8

import 'dart:async';

import 'package:process/process.dart';

import 'base/config.dart';
import 'base/context.dart';
import 'base/error_handling_io.dart';
import 'base/io.dart';
import 'base/logger.dart';
import 'base/os.dart';
import 'base/process.dart';
import 'base/terminal.dart';
import 'base/time.dart';
import 'base/user_messages.dart';
import 'globals.dart' as globals;
import 'persistent_tool_state.dart';
import 'version.dart';

Future<T> runInContext<T>(
  FutureOr<T> Function() runner, {
  Map<Type, Generator> overrides,
}) async {
  // Wrap runner with any asynchronous initialization that should run with the
  // overrides and callbacks.
  FutureOr<T> runnerWrapper() async {
    return runner();
  }

  return context.run<T>(
    name: 'global fallbacks',
    body: runnerWrapper,
    overrides: overrides,
    fallbacks: <Type, Generator>{
      Config: () => Config(
            Config.kFlutterSettings,
            fileSystem: globals.fs,
            logger: globals.logger,
            platform: globals.platform,
          ),
      FlutterVersion: () => FlutterVersion(),
      Logger: () => globals.platform.isWindows
          ? WindowsStdoutLogger(
              terminal: globals.terminal,
              stdio: globals.stdio,
              outputPreferences: globals.outputPreferences,
            )
          : StdoutLogger(
              terminal: globals.terminal,
              stdio: globals.stdio,
              outputPreferences: globals.outputPreferences,
            ),
      OperatingSystemUtils: () => OperatingSystemUtils(
            fileSystem: globals.fs,
            logger: globals.logger,
            platform: globals.platform,
            processManager: globals.processManager,
          ),
      OutputPreferences: () => OutputPreferences(
            wrapText: globals.stdio.hasTerminal ?? false,
            showColor: globals.platform.stdoutSupportsAnsi,
            stdio: globals.stdio,
          ),
      PersistentToolState: () => PersistentToolState(
            fileSystem: globals.fs,
            logger: globals.logger,
            platform: globals.platform,
          ),
      ProcessInfo: () => ProcessInfo(globals.fs),
      ProcessManager: () => ErrorHandlingProcessManager(
            delegate: const LocalProcessManager(),
            platform: globals.platform,
          ),
      ProcessUtils: () => ProcessUtils(
            processManager: globals.processManager,
            logger: globals.logger,
          ),
      ShutdownHooks: () => ShutdownHooks(logger: globals.logger),
      Stdio: () => Stdio(),
      SystemClock: () => const SystemClock(),
      UserMessages: () => UserMessages(),
    },
  );
}
